const express = require("express");
const router = express.Router();
const SimpleGeoSpatialQueryService = require("./SimpleGeoSpatialQueryService.js")
const simpleGeoSpatialQueryService = new SimpleGeoSpatialQueryService();
const pgClient = require("../../../core/pgClient/pgClient.js")



/* 获取某个表的全部的geom数据  */
router.get("/listGeom", (req, res) => {

  let params = req.query;
  simpleGeoSpatialQueryService.listGeom(params).then(result => {
    res.send(result);
  })
})


/* 2024年9月11日 简单-空间查询 */

/* 查询数据库内的矢量数据 */
router.get("/simpleGeoSpatialQuery", (req, res) => {
  let params = req.query;
  simpleGeoSpatialQueryService.SimpleGeoSpatialQuery(params).then(result => {
    res.send(result);
  })
})

/* 2024年9月18日 叠加分析1.0 */
// 相交分析1 Change area
// 功能介绍：根据前端传入的两个表名，返回图表1中与图表2中存在相交的数据（返回图1中的数据），并将面积以及几何数据以GeoJSON 表示。
router.get("/ChangeArea", async (req, res) => {
  // 从请求的查询参数中获取 table1 和 table2 的值
  const { table1, table2 } = req.query;
  // 定义 SQL 查询语句，该查询用于根据空间几何数据计算两个表之间的交集区域及相关信息
  const query = `
  SELECT t1.gid, t1.id, t1.gridcode, t1.objectid, t1.class_name,//返回参数
         ST_Area(ST_Transform(ST_SetSRID(t1.geom, 4326), 32651)) AS area,  // 计算区域面积并转换投影坐标系
         ST_AsGeoJSON(t1.geom) AS st_asgeojson  // 将几何数据转换为 GeoJSON 格式
  FROM "${table1}" AS t1, "${table2}" AS t2  // 从两个表中获取数据
  WHERE ST_Intersects(t1.geom, t2.geom)  // 查找 t1 和 t2 表中的几何数据交集
  `;
  try {
    // 使用 pgClient 执行 SQL 查询，并等待查询结果
    const result = await pgClient.query(query);
    // 查询成功后，将结果以 JSON 格式返回给客户端
    res.json(result.rows);
  } catch (error) {
    console.error("Error in ChangeArea:", error);
    res.status(500).json({ error: "Failed to execute query." });
  }
});

// 相交分析2 No Change area
// 功能介绍：根据前端传入的两个表名，返回图表1中与图表2中不存在相交的数据（返回图1中的数据），并将面积以及几何数据以GeoJSON 表示。
router.get("/NoChangeArea", async (req, res) => {
  const { table1, table2 } = req.query;
  const query = `
  SELECT t1.gid, t1.id, t1.gridcode, t1.objectid,t1.class_name,
        ST_Area(ST_Transform(ST_SetSRID(t1.geom, 4326), 32651)) AS area,
         ST_AsGeoJSON(t1.geom) AS st_asgeojson
  FROM "${table1}" AS t1
  LEFT JOIN "${table2}" AS t2 ON ST_Intersects(t1.geom, t2.geom)
  WHERE t2.geom IS NULL;
`;
  try {
    const result = await pgClient.query(query);
    res.json(result.rows);
  } catch (error) {
    console.error("Error in ChangeArea:", error);
    res.status(500).json({ error: "Failed to execute query." });
  }
});
// 相交分析3 addOverlayAnalysis
// 功能介绍：根据前端传入的两个表名，查询这两个表中几何数据的交集区域（非图1与图2中的数据），并将返回区域的面积以及几何数据的 GeoJSON 表示。
router.get("/addOverlayAnalysis", async (req, res) => {
  const { table1, table2 } = req.query;
  const query = `
WITH valid_t1 AS (  
  SELECT   
    gid,   
    objectid,   
    id,   
    gridcode,   
    geom,  
    class_name,
    CASE   
      WHEN ST_IsValid(geom) THEN geom  
      ELSE ST_MakeValid(geom)
    END AS valid_geom  
  FROM "${table1}"  
  WHERE ST_IsValid(geom) OR ST_MakeValid(geom) IS NOT NULL 
),  
valid_t2 AS (  
  SELECT   
    geom,  
    CASE   
      WHEN ST_IsValid(geom) THEN geom  
      ELSE ST_MakeValid(geom)
    END AS valid_geom  
  FROM "${table2}"  
  WHERE ST_IsValid(geom) OR ST_MakeValid(geom) IS NOT NULL -- 确保至少有一种方式可以得到有效的geom  
)  
SELECT   
  t1.gid,   
  t1.gridcode,   
  t1.id,   
  t1.objectid,   
  t1.class_name,
  ST_AsGeoJSON(ST_Intersection(t1.valid_geom, t2.valid_geom)) AS st_asgeojson,  
  ST_Area(ST_Transform(ST_SetSRID(ST_Intersection(t1.valid_geom, t2.valid_geom), 4326), 32651)) AS area  
FROM valid_t1 AS t1  
INNER JOIN valid_t2 AS t2 ON ST_Intersects(t1.valid_geom, t2.valid_geom);`
  try {
    const result = await pgClient.query(query);
    res.json(result.rows);
  } catch (error) {
    console.error("Error in addOverlayAnalysis:", error);
    res.status(500).json({ error: "Failed to execute query." });
  }
});
// 相交分析4 TypeChangesAnalysis 属性变化检测
// 将 ${table2} 中的 class_name 不等于 ${type} 的数据过滤后，结合 ${table1} 中与其几何数据相交且 class_name 等于 ${type} 的记录，返回符合条件的空间数据及其面积和 GeoJSON 格式。
router.get("/TypeChangesAnalysis", async (req, res) => {
  const { table1, table2, type } = req.query;
  const query = `
WITH filtered_table2 AS (
  SELECT *
  FROM "${table2}"
  WHERE class_name <> '${type}'  -- 过滤掉 table2 中的 class_name = '${type}'
)
SELECT  
    t2.class_name AS class_name,  
    t2.gid AS gid,
    t2.id AS id,
    t2.objectid AS objectid,
    ST_AsGeoJSON(t2.geom) AS st_asgeojson,
    ST_Area(ST_Transform(ST_SetSRID(t2.geom, 4326), 32651)) AS area
FROM
    "${table1}" AS t1  
JOIN  
    filtered_table2 AS t2  
ON  
    ST_Intersects(t1.geom, t2.geom)  
    AND t1.class_name = '${type}'
`;
  try {
    const result = await pgClient.query(query);
    res.json(result.rows);
  } catch (error) {
    console.error("Error in TypeChangesAnalysis:", error);
    res.status(500).json({ error: "Failed to execute query." });
  }
},
  /* 2024年11月09日 叠加分析2.0 */
  // 叠加分析5-从两个不同的表中检索几何数据，并计算每个表中几何数据的面积以及返回几何数据的 GeoJSON 格式。
  router.get("/ChangeQueries", async (req, res) => {
    const { table1, table2 } = req.query;
    const query1 = `
    SELECT gid, id, gridcode, objectid, class_name,
           ST_Area(ST_Transform(ST_SetSRID(geom, 4326), 32651)) AS area,
           ST_AsGeoJSON(geom) AS st_asgeojson
    FROM "${table1}"
    `;
    const query2 = `
    SELECT gid, id, gridcode, objectid, class_name,
           ST_Area(ST_Transform(ST_SetSRID(geom, 4326), 32651)) AS area,
           ST_AsGeoJSON(geom) AS st_asgeojson
    FROM "${table2}"
    `;
    try {
      const result1 = await pgClient.query(query1);
      const result2 = await pgClient.query(query2);
      const combinedResult = {
        table1: result1.rows,
        table2: result2.rows
      };
      res.json(combinedResult);
    } catch (error) {
      console.error("Error in ChangeArea:", error);
      res.status(500).json({ error: "Failed to execute query." });
    }
  }),
  // 相交分析6 差集分析
  // 该查询计算并返回 ${ table2 } 中与 ${ table1 } 相交区域的差异部分的面积和 GeoJSON 格式，按 ${ table2 } 的唯一标识符分组。
  router.get("/NewArea", async (req, res) => {
    const { table1, table2 } = req.query;
    const query = `
     SELECT t2.gid, t2.id, t2.gridcode, t2.objectid,
            ST_Area(ST_Transform(ST_SetSRID(
               ST_Difference(
                  ST_Union(t2.geom),
                  ST_Union(t1.geom)
               ), 4326), 32651)) AS area,
            ST_AsGeoJSON(ST_Difference(
               ST_Union(t2.geom),
               ST_Union(t1.geom)
            )) AS st_asgeojson
     FROM "${table2}" AS t2
     INNER JOIN "${table1}" AS t1 
       ON ST_Intersects(t2.geom, t1.geom)  -- 匹配交集
     GROUP BY t2.gid, t2.id, t2.gridcode, t2.objectid;
   `;
    try {
      const result = await pgClient.query(query);
      res.json(result.rows);
    } catch (error) {
      console.error("Error in ChangeArea:", error);
      res.status(500).json({ error: "Failed to execute query." });
    }
  }),

  // 相交分析7 No Change area
  // 该查询返回 ${table2} 中那些与 ${table1} 没有交集的区域，并计算这些区域的面积及其 GeoJSON 数据。
  router.get("/NoChangeArea_kuangshang", async (req, res) => {
    const { table1, table2 } = req.query;
    const query = `
      SELECT t2.gid, t2.id, t2.gridcode, t2.objectid,
            ST_Area(ST_Transform(ST_SetSRID(t2.geom, 4326), 32651)) AS area,
             ST_AsGeoJSON(t2.geom) AS st_asgeojson
      FROM "${table2}" AS t2
      LEFT JOIN "${table1}" AS t1 ON ST_Intersects(t2.geom, t1.geom)
      WHERE t1.geom IS NULL;
    `;
    try {
      const result = await pgClient.query(query);
      res.json(result.rows);
    } catch (error) {
      console.error("Error in ChangeArea:", error);
      res.status(500).json({ error: "Failed to execute query." });
    }
  }),
  // 相交分析8
  router.get("/ChangeAreaQuery_kuangshang", async (req, res) => {
    const { table1, table2 } = req.query;
    const query = `
  SELECT t1.gid, t1.id, t1.gridcode, t1.objectid,
         ST_Area(ST_Transform(ST_SetSRID(t1.geom, 4326), 32651)) AS area,
         ST_AsGeoJSON(t1.geom) AS st_asgeojson
  FROM "${table1}" AS t1, "${table2}" AS t2
  WHERE ST_Intersects(t1.geom, t2.geom)
`;
    try {
      const result = await pgClient.query(query);
      res.json(result.rows);
    } catch (error) {
      console.error("Error in ChangeArea:", error);
      res.status(500).json({ error: "Failed to execute query." });
    }
  })
);

module.exports = router
