const pgClient = require("../../../core/pgClient/pgClient.js")

class SimpleGeoSpatialQueryDao {
    constructor() {}

    /* 1. 时间限制 */
    _4andTime(startTime, endTime) {
        let andTime = ''
        if (startTime !== '' && endTime !== '') {
            andTime = `and date >= '${startTime}' and date < '${endTime}'`
        } else {
            andTime = ''
        }
        return andTime
    }

    /* ST_Intersects相交查询：线、面、矩形（面） 均适用  （点和缓冲区 采用  buffer）*/
    /**
     * 相交查询（ST_Intersects）
     * @param {*} tbName 图层名（表名）【必填项】
     * @param {*} WKTstr 图形 WKTstr 【必填项】
     * @param {*} startTime 时间起点 default ''
     * @param {*} endTime 时间终点 default ''
     * @param {*} fieldsArr 字段数组 default [* , st_asgeojson(geom)]
     * @param {*} where_condition 额外的限制条件 default ''
     * @returns 查询结果记录
     */
    async ST_Intersects(tbName, WKTstr, startTime = '', endTime = '', fieldsArr = [], where_condition = '') {
        /* 想要查询的字段 */
        let fieldsStr = ""
        if (fieldsArr && fieldsArr.length > 0) {
            fieldsStr = fieldsArr.join(` , `)
        } else {
            fieldsStr = `* , st_asgeojson(geom)`
        }
        /* 时间限制 */
        let andTime = this._4andTime(startTime, endTime)

        // let sql = `select ${fieldsStr} from ${tbName} where ST_Intersects(geom, ST_geomfromtext('${WKTstr}',4326)) ${andTime} ${where_condition} limit 10 offset 0`
        let sql = `select ${fieldsStr} from ${tbName} where ST_Intersects(geom, ST_geomfromtext('${WKTstr}',4326)) ${andTime} ${where_condition}`

        let result = await pgClient.query(sql)

        return result;

    }


    /* 这里做缓冲区查询，不采用st_buffer 无需创建缓冲区对象 效率更高 ,如果想在地图中绘制出来缓冲区的范围，可以采用st_buffer方式*/

    /* ST_DWithin 缓冲区查询 ：点选查询、点、线、面的缓冲区查询 都适合 */

    /**
     * 缓冲区查询-按距离查询（ST_DWithin）
     * @param {*} tbName 图层名（表名）【必填项】
     * @param {*} WKTstr 图形 WKTstr 【必填项】
     * @param {*} distance 距离图形距离 【必填项】
     * @param {*} startTime 时间起点 default ''
     * @param {*} endTime 时间终点 default ''
     * @param {*} fieldsArr 字段数组 default [* , st_asgeojson(geom)]
     * @param {*} where_condition 额外的限制条件 default ''
     * @param {*} where_condition 
     * @returns 
     */
    async ST_DWithin(tbName, WKTstr, distance, startTime = '', endTime = '', fieldsArr = [], where_condition = '') {
        /* 想要查询的字段 */
        let fieldsStr = ""
        if (fieldsArr && fieldsArr.length > 0) {
            fieldsStr = fieldsArr.join(` , `)
        } else {
            fieldsStr = `* , st_asgeojson(geom)`
        }

        /* 时间限制 */
        let andTime = this._4andTime(startTime, endTime)

        let sql = `SELECT ${fieldsStr} 
        FROM ${tbName} 
        WHERE ST_DWithin(geom::geography, ST_geomfromtext('${WKTstr}',4326)::geography, ${distance}) ${andTime} ${where_condition}`;
        // geom 转成 geography 后面的距离distance才是 m
        let result = await pgClient.query(sql)

        return result;

    }


    /* 还有很多种查询方式，但是上面两个函数，基本可以满足所有的空间查询 */



    /* 工具类 */

    /* 1.生成缓冲区对象 （前端可以直接用turfjs）*/
    async ST_Buffer(WKTstr, distance) {

        let sql = `SELECT st_asgeojson( ST_Buffer(ST_GeomFromText('${WKTstr}',4326)::geography , ${distance}, 'endcap=round join=round') );`;
        // geom 转成 geography 后面的距离distance才是 m
        let result = await pgClient.query(sql)

        return result;

    }




    /* 基础类 (只获取geom) */
    async listGeom(tbName) {
        let sql = `select st_asgeojson(geom) from ${tbName}`
        let result = await pgClient.query(sql)
        return result;
    }

    listOther() {

    }


}

module.exports = SimpleGeoSpatialQueryDao